package UI.GameState;

import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;


public class WallState {
    private int frameX, frameY;

    private String source;
    private String[] lines;
    public int numRows, numColumns, w, h;
    private ArrayList<WallData> walls = new ArrayList<>();
    private Image wall1 ;
    private Image wall2 ;
    private Image blank ;


    public WallState(int GAME_WIDTH, int GAME_HEIGHT) {
        frameX = GAME_WIDTH;
        frameY = GAME_HEIGHT;

        try {
            source = new String(Files.readAllBytes(Paths.get("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\maps\\map3.txt")), StandardCharsets.UTF_8);
            wall1 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall1.png"));
            wall2 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\wall2.png"));
            blank = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\blank.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        lines = source.split("\r\n|\r|\n");

        numRows = lines.length;  //number of rows
        numColumns = lines[0].trim().length();  //number of columns

        h = (frameY-30) / (numRows);   //sideY
        w = (frameX-10) / (numColumns);  //sideX

        wall1 = wall1.getScaledInstance(w,h,Image.SCALE_SMOOTH);
        wall2 = wall2.getScaledInstance(w,h,Image.SCALE_SMOOTH);
        blank = blank.getScaledInstance(w,h,Image.SCALE_SMOOTH);

        loadWalls();
    }

    public void paintComponent(Graphics g) {

        g.setColor(Color.WHITE);
        g.fillRect(0, 0, frameX, frameY);
        g.setColor(Color.black);  //colour of the walls

        for (WallData d :walls){
            drawWall(g,d);
        }
    }

    public void loadWalls() {
        // Draw background
        walls.clear();
        int x = 0;
        int y = 0;

        for (int r = 0; r < numRows; r++) {
            x = 0 ;
            String rowChars = lines[r];
            for (int c = 0; c < numColumns; c++) {
                if (rowChars.charAt(c) == '0') {
                    saveBlank( x, y ,3);
                    x += w;
                }
                else if (rowChars.charAt(c) == '1') {
                    saveWalls(x, y, r , c, 1);
                    x += w;
                }
                else if (rowChars.charAt(c) == '2') {
                    saveWalls( x, y, r , c, 2);
                    x += w;
                }
            }
            y += h;
        }
    }



    private void saveWalls(int x,int y , int r , int c ,int image){
        WallData wall = new WallData(x , y ,image,false);
        if ((c<numColumns-1 && lines[r].charAt(c+1)=='1') || (c>0 && lines[r].charAt(c-1)=='1')||(c<numColumns-1 && lines[r].charAt(c+1)=='2') || (c>0 && lines[r].charAt(c-1)=='2')){ //horizontal
            wall.setHorizontal(true);
        }
        if ((r>0 && lines[r-1].charAt(c)=='1') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='1')||(r>0 && lines[r-1].charAt(c)=='2') || (r>0 && r<numRows-1 && lines[r+1].charAt(c)=='2')) { //vertical
            wall.setVertical(true);
        }
        if (lines[r].charAt(c)=='2'){
            wall.setDestructible(true);
        }
        walls.add(wall);
    }

    private void drawWall(Graphics g2d,WallData data){
        Image img;
        if (data.image==1) {
            img = wall1;
        }else if (data.image==2){
            img = wall2;
        }else{
            img = blank;
        }
        g2d.drawImage(img,data.x,data.y,null);
    }

    private void saveBlank(int x,int y,int image){
        WallData wall = new WallData(x , y ,image,true);
        walls.add(wall);
    }

    public WallData getWallForBullet(int x,int y){
        for(WallData d : walls){
            if (d.isWallForBullet(x,y))
                return d;
        }
        return null;
    }


    public boolean isWallForTank(int x,int y){
        for(WallData d : walls){
            if (d.isWallForTank(x,y))
                return true;
        }
        return false;
    }

    public ArrayList<WallData> getWalls() {
        return walls;
    }

    public class WallData {
        private int image;
        private int x,y;
        private boolean horizontal = false;
        private boolean vertical = false;
        private boolean blank;
        private boolean destructible = false;

        public WallData(int x,int y,int image,boolean isBlank){
            this.x = x;
            this.y = y;
            this.image = image;
            this.blank = isBlank;
        }

        public boolean isWallForTank (int x,int y){
            //BUG
            if (blank) return false;
            return (x>this.x-w/2 && x<this.x+w && y>this.y-h/2 && y<this.y+h);
        }

        public boolean isWallForBullet (int x,int y){
            //BUG
            if (blank) return  false;
            return (x>this.x-10 && x<this.x+w+10 && y>this.y-10 && y<this.y+h+10);
        }

        public boolean isHorizontal() {
            return horizontal;
        }

        public void setHorizontal(boolean horizontal) {
            this.horizontal = horizontal;
        }

        public boolean isVertical() {
            return vertical;
        }

        public void setVertical(boolean vertical) {
            this.vertical = vertical;
        }

        public boolean isDestructible() {
            return destructible;
        }

        public void setDestructible(boolean destructible) {
            this.destructible = destructible;
        }

        public int getX() {
            return x;
        }

        public int getY() {
            return y;
        }

    }
}
