package UI.LogIn.Game;

import UI.GameState.*;
import UI.LogIn.Setting;
import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

public class Rendering {

    private Image image1;
    private Image image2;
    private Image explosionImg;
    public static final int GAME_HEIGHT = 720;                  // 720p game resolution
    public static final int GAME_WIDTH = 16 * GAME_HEIGHT / 9;  // wide aspect ratio
    int lastRender = -1;
    private ArrayList<Float> fpsHistory;
    private ArrayList<RewardState> rewards = new ArrayList<>();
    private Setting setting = new Setting();

    public Rendering(){
        fpsHistory = new ArrayList<>(100);
    }

    /**
     * Rendering all game elements based on the game state.
     */
    // Draw all game elements according
    //  to the game 'state' using 'g2d' ...
    public void doRendering(Graphics2D g2d, MyTank myTank, EnemyTank enemyTank) throws IOException {
        // Draw background
        myTank.getWalls().paintComponent(g2d);
        enemyTank.getWalls().paintComponent(g2d);

        // Draw my tank;
        try {
            image1 = ImageIO.read(new File(setting.getTankShapePath()));
        } catch (IOException e) {
            System.out.println(e);
        }
        // Rotation information
        double rotationRequired = Math.toRadians (myTank.rotate);
        BufferedImage image11 = (BufferedImage)image1;
        double locationX = image11.getWidth() / 2;
        double locationY = image11.getHeight() / 2;
        AffineTransform tx = AffineTransform.getRotateInstance(rotationRequired, locationX, locationY);
        AffineTransformOp op = new AffineTransformOp(tx, AffineTransformOp.TYPE_BILINEAR);
        // Drawing the rotated image at the required drawing locations
        int lastX1 = 0;
        int lastY1 = 0;
        if (!myTank.gameOver && !enemyTank.gameOver) {
            g2d.drawImage(op.filter( image11, null),myTank.locX , myTank.locY , null);
            lastX1 = enemyTank.locX;
            lastY1 = enemyTank.locY;
        }
        else
            g2d.drawImage(op.filter( image11, null),lastX1 , lastY1 , null);

        //Draw enemy tank
        try {
            image2 = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\tank_dark.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }
        // Rotation information
        double rotationEnemy = Math.toRadians (enemyTank.rotate);
        BufferedImage image22 = (BufferedImage)image2;
        double locationX2 = image22.getWidth() / 2;
        double locationY2 = image22.getHeight() / 2;
        AffineTransform tx2 = AffineTransform.getRotateInstance(rotationEnemy, locationX2, locationY2);
        AffineTransformOp op2 = new AffineTransformOp(tx2, AffineTransformOp.TYPE_BILINEAR);
        // Drawing the rotated image at the required drawing locations
        int lastX2 = 0;
        int lastY2 = 0;
        if (!myTank.gameOver && !enemyTank.gameOver) {
            g2d.drawImage(op2.filter(image22, null), enemyTank.locX, enemyTank.locY, null);
            lastX2 = enemyTank.locX;
            lastY2 = enemyTank.locY;
        }
        else
            g2d.drawImage(op2.filter( image22, null),lastX2 , lastY2 , null);

        //Draw myTank bullet
        if (myTank.getBullets().size() != 0 && !myTank.gameOver && !enemyTank.gameOver) {
            Iterator<BulletState> it = myTank.getBullets().iterator();
            while (it.hasNext()) {
                BulletState bullet = it.next();
                if (System.currentTimeMillis() - bullet.getCreateTime() > 4000) {
                    it.remove();
                    continue;
                }
                bullet.update();
                g2d.setColor(Color.GRAY);
                g2d.fillOval(bullet.locX, bullet.locY, bullet.diam, bullet.diam);
            }
        }

        //Draw enemy tank bullet
        if (enemyTank.getBullets().size() != 0 && !myTank.gameOver && !enemyTank.gameOver) {
            Iterator<BulletState> it = enemyTank.getBullets().iterator();
            while (it.hasNext()) {
                BulletState bullet = it.next();
                if (System.currentTimeMillis() - bullet.getCreateTime() > 4000) {
                    it.remove();
                    continue;
                }
                bullet.update();
                g2d.setColor(Color.GRAY);
                g2d.fillOval(bullet.locX, bullet.locY, bullet.diam, bullet.diam);
            }
        }

        if (System.currentTimeMillis() - RewardState.startShieldTime > 15000){
            myTank.setHasShield(false);
            enemyTank.setHasShield(false);
        }
        //Draw explosion and ruin the tank
        ArrayList<BulletState> allBullets = new ArrayList<>();
        for (int i = myTank.getBullets().size() - 1 ; i >= 0 ; i -- ){
            if (System.currentTimeMillis() - myTank.getBullets().get(i).getCreateTime() >4000 ){
                break;
            }
            System.out.println(i);
            allBullets.add(myTank.getBullets().get(i));
        }
        for (int i = enemyTank.getBullets().size() - 1 ; i >= 0 ; i -- ){
            if (System.currentTimeMillis() - enemyTank.getBullets().get(i).getCreateTime() >4000 ){
                break;
            }
            allBullets.add(enemyTank.getBullets().get(i));
        }
        try {
            explosionImg = ImageIO.read(new File("C:\\Users\\Lenovo\\IdeaProjects\\JTankTrouble\\pics\\explosion3.png"));
        } catch (IOException e) {
            System.out.println(e);
        }
        long startExploding ;
        for (BulletState bullet : allBullets){//check my tank
            if ((bullet.locX >myTank.locX - 20 && bullet.locX <myTank.locX + 20 && bullet.locY >myTank.locY - 10&& bullet.locY <myTank.locY + 10)&& !myTank.hasShield() && System.currentTimeMillis() -  bullet.getCreateTime() >100){
                startExploding = System.currentTimeMillis();
                if (System.currentTimeMillis() - startExploding < 1000) {
                    g2d.drawImage(op2.filter((BufferedImage) explosionImg, null), myTank.locX, myTank.locY, null);
                    myTank.gameOver = true;
                }
            }
        }
        for (BulletState bullet : allBullets){//check enemy tank
            if ((bullet.locX >enemyTank.locX - 20 && bullet.locX <enemyTank.locX + 20 && bullet.locY >enemyTank.locY - 10&& bullet.locY <enemyTank.locY + 10)&& !enemyTank.hasShield()&& System.currentTimeMillis() -  bullet.getCreateTime() >100){
                startExploding = System.currentTimeMillis();
                System.out.println("ex");
                if (System.currentTimeMillis() - startExploding < 1000) {
                    g2d.drawImage(op2.filter((BufferedImage) explosionImg, null), enemyTank.locX, enemyTank.locY, null);
                    enemyTank.gameOver = true;

                }
            }
        }

        //Draw rewards
        boolean newReward = (System.currentTimeMillis() -GameLoop.startTime >= 7000);
        if (newReward) {
            GameLoop.startTime = System.currentTimeMillis();
            RewardState reward = new RewardState(myTank.getWalls());
            reward.setVisible(true);
            rewards.add(reward);
        }

        if (rewards.size() != 0){
            Iterator<RewardState> it = rewards.iterator();
            while (it.hasNext()){
                RewardState reward = it.next();
                if (reward.isVisible() && reward.isRewardForTank(myTank.locX , myTank.locY)){
                    reward.runReward(myTank);
                    reward.setVisible(false);
                    it.remove();
                }
                else if (reward.isVisible() &&reward.isRewardForTank(enemyTank.locX , enemyTank.locY)){
                    reward.runReward(enemyTank);
                    reward.setVisible(false);
                    it.remove();
                }
                else if (reward.isVisible()){
                    g2d.drawImage(reward.getRewardImage(),reward.locX, reward.locY, null);
                }
            }
        }


        // Print FPS info
        long currentRender = System.currentTimeMillis();
        if (lastRender > 0) {
            fpsHistory.add(1000.0f / (currentRender - lastRender));
            if (fpsHistory.size() > 100) {
                fpsHistory.remove(0); // remove oldest
            }
            float avg = 0.0f;
            for (float fps : fpsHistory) {
                avg += fps;
            }
            avg /= fpsHistory.size();
            String str = String.format("Average FPS = %.1f , Last Interval = %d ms",
                    avg, (currentRender - lastRender));
            g2d.setColor(Color.CYAN);
            g2d.setFont(g2d.getFont().deriveFont(18.0f));
            int strWidth = g2d.getFontMetrics().stringWidth(str);
            int strHeight = g2d.getFontMetrics().getHeight();
            g2d.drawString(str, (GAME_WIDTH - strWidth) / 2, strHeight + 50);
        }
        lastRender = (int) currentRender;
        // Print user guide
        String userGuide
                = "Use the MOUSE or ARROW KEYS to move the TANK. "
                + "Press ESCAPE to end the game.";
        g2d.setFont(g2d.getFont().deriveFont(18.0f));
        g2d.drawString(userGuide, 10, GAME_HEIGHT - 10);
        // Draw GAME OVER
        if (myTank.gameOver) {
            String str = "GAME OVER";
            g2d.setColor(Color.WHITE);
            g2d.setFont(g2d.getFont().deriveFont(Font.BOLD).deriveFont(64.0f));
            int strWidth = g2d.getFontMetrics().stringWidth(str);
            g2d.drawString(str, (GAME_WIDTH - strWidth) / 2, GAME_HEIGHT / 2);
        }
    }
}
